/*+*****************************************************************************
*                                                                              *
* File: mc.c                                                                   *
*                                                                              *
* Description: memory chunk handling                                           *
*                                                                              *
**-****************************************************************************/

#ifndef __lint
char mc_vers[] = "@(#)mc.c	1.3	01/05/98	Written by Lionel Cons";
#endif /* __lint */

/******* Include Files ********************************************************/

#include "mc.h"
#include "util.h"

/******* Constants ************************************************************/

#define MC_MIN_SIZE	64	/* minimum size of a memory chunk */

/******* Macros ***************************************************************/

/******* External Stuff *******************************************************/

/******* Local Stuff **********************************************************/

/*
 * extended memory chunk
 */
typedef struct MCX_ {
  /* same as MC */
  char *base;
  int len;
  MC *next;
  /* extra internal data */
  char *dbuf;			/* dynamic buffer */
  char sbuf[MC_MIN_SIZE];	/* static buffer */
} MCX;

static MC *mc_first = NULL;	/* linked list of free memory chunks */
#ifdef DEBUG
static int mc_count = 0;	/* number of memory chunks allocated */
#endif

/*
 * give a new memory chunk of the given size
 */
MC *mc_new(int need)
{
  char *buf;
  MCX *mcxp;

  /* find buffer */
  if (need > MC_MIN_SIZE) {
    /* we need a big chunk, we must allocate dbuf */
    buf = (char *)safe_malloc(need);
#ifdef DEBUG
    if (DEBUGGING(DBG_MC)) dprintf(NULL, "dynamic memory chunk needed: %d", need);
#endif
  } else {
    /* we need a small chunk, sbuf is enough */
    buf = NULL;
#ifdef DEBUG
    if (DEBUGGING(DBG_MC)) dprintf(NULL, "static memory chunk needed: %d", need);
#endif
  }
  /* find chunk */
  if (mc_first) {
    /* we have a free chunk */
    mcxp = (MCX *)mc_first;
    mc_first = mcxp->next;
  } else {
    /* we must allocate one */
    mcxp = NEW(MCX);
#ifdef DEBUG
    if (DEBUGGING(DBG_MC)) dprintf(NULL, "new memory chunk allocated: %d", ++mc_count);
#endif
  }
  /* initialise chunk */
  mcxp->base = (buf ? buf : mcxp->sbuf);
  mcxp->len = need;
  mcxp->next = NULL;
  mcxp->dbuf = buf;
  /* return it */
  return((MC *)mcxp);
}

/*
 * get rid of a memory chunk
 */
void mc_old(MC *mcp)
{
  MCX *mcxp = (MCX *)mcp;

  /* free dynamic buffer if any */
  if (mcxp->dbuf) free(mcxp->dbuf);
  /* queue the chunk */
  mcp->next = mc_first;
  mc_first = mcp;
}
