/*+*****************************************************************************
*                                                                              *
* File: popup.c                                                                *
*                                                                              *
* Description: handle the popup menu                                           *
*                                                                              *
**-****************************************************************************/

#ifndef __lint
char popup_vers[] = "@(#)popup.c	1.5	01/16/98	Written by Lionel Cons";
#endif /* __lint */

/******* Include Files ********************************************************/

#include <time.h>
#include <X11/Xlib.h>
#include <Xm/LabelG.h>
#include <Xm/PushBG.h>
#include <Xm/SeparatoG.h>
#include <Xm/ToggleBG.h>
#include <Xm/RowColumn.h>

#include "popup.h"
#include "conn.h"
#include "input.h"
#include "xu.h"
#include "xp.h"

/******* Constants ************************************************************/

#define SCRATCH_SIZE	2048	   /* size of the area used to build messages */

/******* Macros ***************************************************************/

/******* External Stuff *******************************************************/

/******* Local Stuff **********************************************************/

static void (*popup_kill_connection)(CONN *);
static char popup_scratch[SCRATCH_SIZE];

/*
 * callback to change the frozen flag
 */
/*ARGSUSED*/
static void popup_frozen_cb(Widget w, CONN *c, XmToggleButtonCallbackStruct *cbs)
{
  BITCHG(c->flags, CF_FROZEN);
  /* update the input */
  input_update_connection(c);
}

/*
 * callback to change the checked flag
 */
/*ARGSUSED*/
static void popup_checked_cb(Widget w, CONN *c, XmToggleButtonCallbackStruct *cbs)
{
  BITCHG(c->flags, CF_CHECKED);
}

/*
 * callback to change the safe flag
 */
/*ARGSUSED*/
static void popup_safe_cb(Widget w, CONN *c, XmToggleButtonCallbackStruct *cbs)
{
  BITCHG(c->flags, CF_SAFE);
}

/*
 * dismiss the info box
 */
static void popup_info_dismiss(Widget w, CONN *c)
{
  xu_dismiss(w, NULL);
  c->ui.info = (Widget)NULL;
}

/*
 * put the text to display in a static buffer
 */
static char *popup_info_text(CONN *c)
{
  int n, i;
  char *cp;

  /* init */
  cp = popup_scratch;

  /* basic info */
  sprintf(cp, "Connection %s from %s [%s]\n", c->id, c->name, c->host->ip_name);
  cp += strlen(cp);
  sprintf(cp, "Started on: %s", ctime(&c->start));
  cp += strlen(cp);
  sprintf(cp, "Idle since: %s", ctime(&c->idle_since));
  cp += strlen(cp);
  sprintf(cp, PRId32BITS " X packets (" PRId32BITS " KB) with %d alert%s\n",
	  c->xinfo.pkt_count, (c->xinfo.pkt_size >> 10),
	  c->xinfo.alert_count, (c->xinfo.alert_count > 1 ? "s" : ""));
  cp += strlen(cp);
  sprintf(cp, "Resource id base " PRIx32BITS ", mask " PRIx32BITS "\n",
	  c->xinfo.resid_base, c->xinfo.resid_mask);
  cp += strlen(cp);

  /* windows */
  n = xu_scan(c->xinfo.resid_base, c->xinfo.resid_mask);
  if (n == 0) {
    sprintf(cp, "No window displayed\n");
    cp += strlen(cp);
  } else {
    sprintf(cp, "Window%s:\n", (n > 1 ? "s" : ""));
    cp += strlen(cp);
    for (i=0; i<n; i++) {
      if (strlen(xu_window_info[i].name) + 60 > SCRATCH_SIZE - (int)(cp - popup_scratch)) {
	/* oops, not enough space in scratch */
	strcpy(cp, "   ...\n");
	break;
      }
      sprintf(cp, "   " PRIx32BITS " %s (%s)\n", xu_window_info[i].id,
	      xu_window_info[i].name, xu_window_info[i].geom);
      cp += strlen(cp);
    }
  }
  
  return(popup_scratch);
}

/*
 * callback to show the info box
 */
/*ARGSUSED*/
static void popup_info_cb(Widget w, CONN *c, XmPushButtonCallbackStruct *cbs)
{
  ANSWER ok;
  char *text;

  /* init */
  text = popup_info_text(c);

  /* already displayed? */
  if (c->ui.info) {
    xu_text(c->ui.info, text);
    xu_raise(c->ui.info);
    return;
  }

  /* display it */
  ok.text = "Ok";
  ok.callback = (XtCallbackProc)popup_info_dismiss;
  ok.data = (XtPointer)c;
  c->ui.info = xu_info(text, &ok);
}

/*
 * callback to confirm a connection
 */
/*ARGSUSED*/
static void popup_kill_cb(Widget w, CONN *c, XmPushButtonCallbackStruct *cbs)
{
  popup_kill_connection(c);
}

/*
 * event handler for the popup menu
 */
void popup_handler(Widget w, CONN *c, XButtonPressedEvent *event)
{
  /* create menu if needed */
  if (c->ui.menu == NULL) {
    Arg args[2];
    Widget item;

    c->ui.menu = XmCreatePopupMenu(w, "Popup Menu", args, 0);
    item = XmCreateLabelGadget(c->ui.menu, c->id, args, 0);
    XtManageChild(item);
    XtSetArg(args[0], XmNseparatorType, XmDOUBLE_LINE);
    item = XmCreateSeparatorGadget(c->ui.menu, "sep 1", args, 1);
    XtManageChild(item);
    item = XmCreateToggleButtonGadget(c->ui.menu, "Frozen", args, 0);
    XtManageChild(item);
    XtAddCallback(item, XmNvalueChangedCallback,
		(XtCallbackProc)popup_frozen_cb, (XtPointer)c);
    c->ui.menu_frozen = item;
    item = XmCreateToggleButtonGadget(c->ui.menu, "Checked", args, 0);
    XtManageChild(item);
    XtAddCallback(item, XmNvalueChangedCallback,
		(XtCallbackProc)popup_checked_cb, (XtPointer)c);
    c->ui.menu_checked = item;
    item = XmCreateToggleButtonGadget(c->ui.menu, "Safe", args, 0);
    XtManageChild(item);
    XtAddCallback(item, XmNvalueChangedCallback,
		(XtCallbackProc)popup_safe_cb, (XtPointer)c);
    c->ui.menu_safe = item;
    XtSetArg(args[0], XmNseparatorType, XmSINGLE_LINE);
    item = XmCreateSeparatorGadget(c->ui.menu, "sep 2", args, 1);
    XtManageChild(item);
    item = XmCreatePushButtonGadget(c->ui.menu, "Info...", args, 0);
    XtManageChild(item);
    XtAddCallback(item, XmNactivateCallback,
		(XtCallbackProc)popup_info_cb, (XtPointer)c);
    item = XmCreatePushButtonGadget(c->ui.menu, "Kill...", args, 0);
    XtManageChild(item);
    XtAddCallback(item, XmNactivateCallback,
		(XtCallbackProc)popup_kill_cb, (XtPointer)c);
  }

  /* adjust menu state and display it */
  XtVaSetValues(c->ui.menu_frozen,
		XmNset, (BITTST(c->flags, CF_FROZEN) ? True : False),
		NULL);
  XtVaSetValues(c->ui.menu_checked,
		XmNset, (BITTST(c->flags, CF_CHECKED) ? True : False),
		NULL);
  XtVaSetValues(c->ui.menu_safe,
		XmNset, (BITTST(c->flags, CF_SAFE) ? True : False),
		NULL);
  XmMenuPosition(c->ui.menu, event);
  XtManageChild(c->ui.menu);
}

/*
 * setup
 */
void popup_setup(void (*kc)(CONN *))
{
  popup_kill_connection = kc;
}
