/*+*****************************************************************************
*                                                                              *
* File: xp.h                                                                   *
*                                                                              *
**-****************************************************************************/

/*
 * @(#)xp.h	1.6	01/07/98	Written by Lionel Cons
 */

#ifndef _xp_h_
#define _xp_h_

/******* Include Files ********************************************************/

#include "mc.h"
#include "xmd.h"

/******* Constants ************************************************************/

/*
 * known things that can be printed by xp_print
 */
#define XP_CSETUP	'c'	/* client setup */
#define XP_REQUEST	'r'	/* client request */
#define XP_SSETUP	's'	/* server setup */
#define XP_PACKET	'p'	/* server packet */

/*
 * alert levels
 */
#define AL_INTERNAL	'I'	/* request that may cause internal problem */
#define AL_DUBIOUS	'D'	/* possible security incident */
#define AL_SEVERE	'S'	/* real security incident */

/******* Macros ***************************************************************/

/******* Types ****************************************************************/

typedef struct XINFO_ {
  /* X info */
  U8BITS  sex;                  /* byte-sex */
  U16BITS seq;                  /* sequence number of last request */
  U32BITS resid_base;		/* resource id base */
  U32BITS resid_mask;		/* resource id mask */
  U32BITS sel_req;		/* selection requestor */
  U32BITS sel_prop;		/* selection property */
  U32BITS pkt_count;		/* number of packets exchanged */
  U32BITS pkt_size;		/* size of packets exchanged */
  /* alert info */
  char *alert;			/* alert (reason) or NULL if there is none */
  char  alert_level;		/* alert level */
  int   alert_count;		/* number of alerts detected */
  /* fake requests and packets */
  MC *fake_request;		/* fake client request */
  MC *fake_packet;		/* fake server packet */
} XINFO;

/******* Variables ************************************************************/

/******* Functions ************************************************************/

void xp_init(XINFO *);
void xp_noop(char *, XINFO *);
void xp_check(char *, XINFO *);
void xp_trace(char, char *, char *, XINFO *);
#ifdef DEBUG
void xp_print(char, char *, char *, XINFO *);
#endif

#endif /* _xp_h_ */
/* DON'T ADD ANYTHING AFTER THIS LINE */
